@@ -11,8 +11,10 @@ import (
 	dockeracquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/docker"
 	fileacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/file"
 	journalctlacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/journalctl"
+	kafkaacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/kafka"
 	kinesisacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/kinesis"
 	syslogacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/syslog"
+	wineventlogacquisition "github.com/crowdsecurity/crowdsec/pkg/acquisition/modules/wineventlog"
 	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
 	"github.com/crowdsecurity/crowdsec/pkg/types"
 	"github.com/pkg/errors"
@@ -65,6 +67,14 @@ var AcquisitionSources = []struct {
 		name:  "kinesis",
 		iface: func() DataSource { return &kinesisacquisition.KinesisSource{} },
 	},
+	{
+		name:  "wineventlog",
+		iface: func() DataSource { return &wineventlogacquisition.WinEventLogSource{} },
+	},
+	{
+		name:  "kafka",
+		iface: func() DataSource { return &kafkaacquisition.KafkaSource{} },
+	},
 }
 
 func GetDataSourceIface(dataSourceType string) DataSource {
@@ -165,12 +175,13 @@ func LoadAcquisitionFromFile(config *csconfig.CrowdsecServiceCfg) ([]DataSource,
 		log.Infof("loading acquisition file : %s", acquisFile)
 		yamlFile, err := os.Open(acquisFile)
 		if err != nil {
-			return nil, errors.Wrapf(err, "can't open %s", acquisFile)
+			return nil, err
 		}
 		dec := yaml.NewDecoder(yamlFile)
 		dec.SetStrict(true)
 		for {
 			var sub configuration.DataSourceCommonCfg
+			var idx int
 			err = dec.Decode(&sub)
 			if err != nil {
 				if err == io.EOF {
@@ -188,21 +199,23 @@ func LoadAcquisitionFromFile(config *csconfig.CrowdsecServiceCfg) ([]DataSource,
 			if len(sub.Labels) == 0 {
 				if sub.Source == "" {
 					log.Debugf("skipping empty item in %s", acquisFile)
+					idx += 1
 					continue
 				}
-				return nil, fmt.Errorf("missing labels in %s", acquisFile)
+				return nil, fmt.Errorf("missing labels in %s (position: %d)", acquisFile, idx)
 			}
 			if sub.Source == "" {
-				return nil, fmt.Errorf("data source type is empty ('source') in %s", acquisFile)
+				return nil, fmt.Errorf("data source type is empty ('source') in %s (position: %d)", acquisFile, idx)
 			}
 			if GetDataSourceIface(sub.Source) == nil {
-				return nil, fmt.Errorf("unknown data source %s in %s", sub.Source, acquisFile)
+				return nil, fmt.Errorf("unknown data source %s in %s (position: %d)", sub.Source, acquisFile, idx)
 			}
 			src, err := DataSourceConfigure(sub)
 			if err != nil {
-				return nil, errors.Wrapf(err, "while configuring datasource of type %s from %s", sub.Source, acquisFile)
+				return nil, errors.Wrapf(err, "while configuring datasource of type %s from %s (position: %d)", sub.Source, acquisFile, idx)
 			}
 			sources = append(sources, *src)
+			idx += 1
 		}
 	}
 	return sources, nil
@@ -218,11 +231,10 @@ func GetMetrics(sources []DataSource, aggregated bool) error {
 		}
 		for _, metric := range metrics {
 			if err := prometheus.Register(metric); err != nil {
-				if _, ok := err.(prometheus.AlreadyRegisteredError); ok {
-					//ignore the error
-				} else {
+				if _, ok := err.(prometheus.AlreadyRegisteredError); !ok {
 					return errors.Wrapf(err, "could not register metrics for datasource %s", sources[i].GetName())
 				}
+				//ignore the error
 			}
 		}
 
