@@ -3,10 +3,10 @@ package apiserver
 import (
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
 	"os"
+	"path/filepath"
 	"strings"
 	"testing"
 	"time"
@@ -16,6 +16,7 @@ import (
 	"github.com/crowdsecurity/crowdsec/pkg/models"
 	"github.com/crowdsecurity/crowdsec/pkg/types"
 	"github.com/go-openapi/strfmt"
+	"github.com/pkg/errors"
 
 	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
 	"github.com/crowdsecurity/crowdsec/pkg/database"
@@ -33,6 +34,7 @@ var MachineTest = models.WatcherAuthRequest{
 }
 
 var UserAgent = fmt.Sprintf("crowdsec-test/%s", cwversion.Version)
+var emptyBody = strings.NewReader("")
 
 func LoadTestConfig() csconfig.Config {
 	config := csconfig.Config{}
@@ -40,9 +42,10 @@ func LoadTestConfig() csconfig.Config {
 	flushConfig := csconfig.FlushDBCfg{
 		MaxAge: &maxAge,
 	}
+	tempDir, _ := os.MkdirTemp("", "crowdsec_tests")
 	dbconfig := csconfig.DatabaseCfg{
 		Type:   "sqlite",
-		DbPath: "./ent",
+		DbPath: filepath.Join(tempDir, "ent"),
 		Flush:  &flushConfig,
 	}
 	apiServerConfig := csconfig.LocalApiServerCfg{
@@ -71,9 +74,10 @@ func LoadTestConfigForwardedFor() csconfig.Config {
 	flushConfig := csconfig.FlushDBCfg{
 		MaxAge: &maxAge,
 	}
+	tempDir, _ := os.MkdirTemp("", "crowdsec_tests")
 	dbconfig := csconfig.DatabaseCfg{
 		Type:   "sqlite",
-		DbPath: "./ent",
+		DbPath: filepath.Join(tempDir, "ent"),
 		Flush:  &flushConfig,
 	}
 	apiServerConfig := csconfig.LocalApiServerCfg{
@@ -98,58 +102,57 @@ func LoadTestConfigForwardedFor() csconfig.Config {
 	return config
 }
 
-func NewAPIServer() (*APIServer, error) {
+func NewAPIServer() (*APIServer, csconfig.Config, error) {
 	config := LoadTestConfig()
 	os.Remove("./ent")
 	apiServer, err := NewServer(config.API.Server)
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
 	log.Printf("Creating new API server")
 	gin.SetMode(gin.TestMode)
-	return apiServer, nil
+	return apiServer, config, nil
 }
 
-func NewAPITest() (*gin.Engine, error) {
-	apiServer, err := NewAPIServer()
+func NewAPITest() (*gin.Engine, csconfig.Config, error) {
+	apiServer, config, err := NewAPIServer()
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
 	err = apiServer.InitController()
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
 	router, err := apiServer.Router()
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
-	return router, nil
+	return router, config, nil
 }
 
-func NewAPITestForwardedFor() (*gin.Engine, error) {
+func NewAPITestForwardedFor() (*gin.Engine, csconfig.Config, error) {
 	config := LoadTestConfigForwardedFor()
 
 	os.Remove("./ent")
 	apiServer, err := NewServer(config.API.Server)
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
 	err = apiServer.InitController()
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
 	log.Printf("Creating new API server")
 	gin.SetMode(gin.TestMode)
 	router, err := apiServer.Router()
 	if err != nil {
-		return nil, fmt.Errorf("unable to run local API: %s", err)
+		return nil, config, fmt.Errorf("unable to run local API: %s", err)
 	}
-	return router, nil
+	return router, config, nil
 }
 
-func ValidateMachine(machineID string) error {
-	config := LoadTestConfig()
-	dbClient, err := database.NewClient(config.API.Server.DbConfig)
+func ValidateMachine(machineID string, config *csconfig.DatabaseCfg) error {
+	dbClient, err := database.NewClient(config)
 	if err != nil {
 		return fmt.Errorf("unable to create new database client: %s", err)
 	}
@@ -159,9 +162,8 @@ func ValidateMachine(machineID string) error {
 	return nil
 }
 
-func GetMachineIP(machineID string) (string, error) {
-	config := LoadTestConfig()
-	dbClient, err := database.NewClient(config.API.Server.DbConfig)
+func GetMachineIP(machineID string, config *csconfig.DatabaseCfg) (string, error) {
+	dbClient, err := database.NewClient(config)
 	if err != nil {
 		return "", fmt.Errorf("unable to create new database client: %s", err)
 	}
@@ -177,6 +179,79 @@ func GetMachineIP(machineID string) (string, error) {
 	return "", nil
 }
 
+func GetAlertReaderFromFile(path string) *strings.Reader {
+
+	alertContentBytes, err := os.ReadFile(path)
+	if err != nil {
+		log.Fatal(err)
+	}
+
+	alerts := make([]*models.Alert, 0)
+	if err := json.Unmarshal(alertContentBytes, &alerts); err != nil {
+		log.Fatal(err)
+	}
+
+	for _, alert := range alerts {
+		*alert.StartAt = time.Now().UTC().Format(time.RFC3339)
+		*alert.StopAt = time.Now().UTC().Format(time.RFC3339)
+	}
+
+	alertContent, err := json.Marshal(alerts)
+	if err != nil {
+		log.Fatal(err)
+	}
+	return strings.NewReader(string(alertContent))
+
+}
+
+func readDecisionsGetResp(resp *httptest.ResponseRecorder) ([]*models.Decision, int, error) {
+	var response []*models.Decision
+	if resp == nil {
+		return nil, 0, errors.New("response is nil")
+	}
+	err := json.Unmarshal(resp.Body.Bytes(), &response)
+	if err != nil {
+		return nil, resp.Code, err
+	}
+	return response, resp.Code, nil
+}
+
+func readDecisionsErrorResp(resp *httptest.ResponseRecorder) (map[string]string, int, error) {
+	var response map[string]string
+	if resp == nil {
+		return nil, 0, errors.New("response is nil")
+	}
+	err := json.Unmarshal(resp.Body.Bytes(), &response)
+	if err != nil {
+		return nil, resp.Code, err
+	}
+	return response, resp.Code, nil
+}
+
+func readDecisionsDeleteResp(resp *httptest.ResponseRecorder) (*models.DeleteDecisionResponse, int, error) {
+	var response models.DeleteDecisionResponse
+	if resp == nil {
+		return nil, 0, errors.New("response is nil")
+	}
+	err := json.Unmarshal(resp.Body.Bytes(), &response)
+	if err != nil {
+		return nil, resp.Code, err
+	}
+	return &response, resp.Code, nil
+}
+
+func readDecisionsStreamResp(resp *httptest.ResponseRecorder) (map[string][]*models.Decision, int, error) {
+	response := make(map[string][]*models.Decision)
+	if resp == nil {
+		return nil, 0, errors.New("response is nil")
+	}
+	err := json.Unmarshal(resp.Body.Bytes(), &response)
+	if err != nil {
+		return nil, resp.Code, err
+	}
+	return response, resp.Code, nil
+}
+
 func CreateTestMachine(router *gin.Engine) (string, error) {
 	b, err := json.Marshal(MachineTest)
 	if err != nil {
@@ -185,16 +260,14 @@ func CreateTestMachine(router *gin.Engine) (string, error) {
 	body := string(b)
 
 	w := httptest.NewRecorder()
-	req, _ := http.NewRequest("POST", "/v1/watchers", strings.NewReader(body))
+	req, _ := http.NewRequest(http.MethodPost, "/v1/watchers", strings.NewReader(body))
 	req.Header.Set("User-Agent", UserAgent)
 	router.ServeHTTP(w, req)
 	return body, nil
 }
 
-func CreateTestBouncer() (string, error) {
-	config := LoadTestConfig()
-
-	dbClient, err := database.NewClient(config.API.Server.DbConfig)
+func CreateTestBouncer(config *csconfig.DatabaseCfg) (string, error) {
+	dbClient, err := database.NewClient(config)
 	if err != nil {
 		log.Fatalf("unable to create new database client: %s", err)
 	}
@@ -202,7 +275,7 @@ func CreateTestBouncer() (string, error) {
 	if err != nil {
 		return "", fmt.Errorf("unable to generate api key: %s", err)
 	}
-	err = dbClient.CreateBouncer("test", "127.0.0.1", middlewares.HashSHA512(apiKey))
+	_, err = dbClient.CreateBouncer("test", "127.0.0.1", middlewares.HashSHA512(apiKey), types.ApiKeyAuthType)
 	if err != nil {
 		return "", fmt.Errorf("unable to create blocker: %s", err)
 	}
@@ -216,7 +289,7 @@ func TestWithWrongDBConfig(t *testing.T) {
 	apiServer, err := NewServer(config.API.Server)
 
 	assert.Equal(t, apiServer, &APIServer{})
-	assert.Equal(t, "unable to init database client: unknown database type", err.Error())
+	assert.Equal(t, "unable to init database client: unknown database type 'test'", err.Error())
 }
 
 func TestWithWrongFlushConfig(t *testing.T) {
@@ -230,13 +303,13 @@ func TestWithWrongFlushConfig(t *testing.T) {
 }
 
 func TestUnknownPath(t *testing.T) {
-	router, err := NewAPITest()
+	router, _, err := NewAPITest()
 	if err != nil {
 		log.Fatalf("unable to run local API: %s", err)
 	}
 
 	w := httptest.NewRecorder()
-	req, _ := http.NewRequest("GET", "/test", nil)
+	req, _ := http.NewRequest(http.MethodGet, "/test", nil)
 	req.Header.Set("User-Agent", UserAgent)
 	router.ServeHTTP(w, req)
 
@@ -266,27 +339,25 @@ func TestLoggingDebugToFileConfig(t *testing.T) {
 	flushConfig := csconfig.FlushDBCfg{
 		MaxAge: &maxAge,
 	}
+	tempDir, _ := os.MkdirTemp("", "crowdsec_tests")
 	dbconfig := csconfig.DatabaseCfg{
 		Type:   "sqlite",
-		DbPath: "./ent",
+		DbPath: filepath.Join(tempDir, "ent"),
 		Flush:  &flushConfig,
 	}
 	cfg := csconfig.LocalApiServerCfg{
 		ListenURI: "127.0.0.1:8080",
 		LogMedia:  "file",
-		LogDir:    ".",
+		LogDir:    tempDir,
 		DbConfig:  &dbconfig,
 	}
 	lvl := log.DebugLevel
-	expectedFile := "./crowdsec_api.log"
+	expectedFile := fmt.Sprintf("%s/crowdsec_api.log", tempDir)
 	expectedLines := []string{"/test42"}
 	cfg.LogLevel = &lvl
 
-	os.Remove("./crowdsec.log")
-	os.Remove(expectedFile)
-
 	// Configure logging
-	if err := types.SetDefaultLoggerConfig(cfg.LogMedia, cfg.LogDir, *cfg.LogLevel, cfg.LogMaxSize, cfg.LogMaxFiles, cfg.LogMaxAge, cfg.CompressLogs); err != nil {
+	if err := types.SetDefaultLoggerConfig(cfg.LogMedia, cfg.LogDir, *cfg.LogLevel, cfg.LogMaxSize, cfg.LogMaxFiles, cfg.LogMaxAge, cfg.CompressLogs, false); err != nil {
 		t.Fatal(err.Error())
 	}
 	api, err := NewServer(&cfg)
@@ -298,7 +369,7 @@ func TestLoggingDebugToFileConfig(t *testing.T) {
 	}
 
 	w := httptest.NewRecorder()
-	req, _ := http.NewRequest("GET", "/test42", nil)
+	req, _ := http.NewRequest(http.MethodGet, "/test42", nil)
 	req.Header.Set("User-Agent", UserAgent)
 	api.router.ServeHTTP(w, req)
 	assert.Equal(t, 404, w.Code)
@@ -306,7 +377,7 @@ func TestLoggingDebugToFileConfig(t *testing.T) {
 	time.Sleep(500 * time.Millisecond)
 
 	//check file content
-	data, err := ioutil.ReadFile(expectedFile)
+	data, err := os.ReadFile(expectedFile)
 	if err != nil {
 		t.Fatalf("failed to read file : %s", err)
 	}
@@ -317,9 +388,6 @@ func TestLoggingDebugToFileConfig(t *testing.T) {
 		}
 	}
 
-	os.Remove("./crowdsec.log")
-	os.Remove(expectedFile)
-
 }
 
 func TestLoggingErrorToFileConfig(t *testing.T) {
@@ -329,26 +397,24 @@ func TestLoggingErrorToFileConfig(t *testing.T) {
 	flushConfig := csconfig.FlushDBCfg{
 		MaxAge: &maxAge,
 	}
+	tempDir, _ := os.MkdirTemp("", "crowdsec_tests")
 	dbconfig := csconfig.DatabaseCfg{
 		Type:   "sqlite",
-		DbPath: "./ent",
+		DbPath: filepath.Join(tempDir, "ent"),
 		Flush:  &flushConfig,
 	}
 	cfg := csconfig.LocalApiServerCfg{
 		ListenURI: "127.0.0.1:8080",
 		LogMedia:  "file",
-		LogDir:    ".",
+		LogDir:    tempDir,
 		DbConfig:  &dbconfig,
 	}
 	lvl := log.ErrorLevel
-	expectedFile := "./crowdsec_api.log"
+	expectedFile := fmt.Sprintf("%s/crowdsec_api.log", tempDir)
 	cfg.LogLevel = &lvl
 
-	os.Remove("./crowdsec.log")
-	os.Remove(expectedFile)
-
 	// Configure logging
-	if err := types.SetDefaultLoggerConfig(cfg.LogMedia, cfg.LogDir, *cfg.LogLevel, cfg.LogMaxSize, cfg.LogMaxFiles, cfg.LogMaxAge, cfg.CompressLogs); err != nil {
+	if err := types.SetDefaultLoggerConfig(cfg.LogMedia, cfg.LogDir, *cfg.LogLevel, cfg.LogMaxSize, cfg.LogMaxFiles, cfg.LogMaxAge, cfg.CompressLogs, false); err != nil {
 		t.Fatal(err.Error())
 	}
 	api, err := NewServer(&cfg)
@@ -360,7 +426,7 @@ func TestLoggingErrorToFileConfig(t *testing.T) {
 	}
 
 	w := httptest.NewRecorder()
-	req, _ := http.NewRequest("GET", "/test42", nil)
+	req, _ := http.NewRequest(http.MethodGet, "/test42", nil)
 	req.Header.Set("User-Agent", UserAgent)
 	api.router.ServeHTTP(w, req)
 	assert.Equal(t, 404, w.Code)
@@ -368,12 +434,11 @@ func TestLoggingErrorToFileConfig(t *testing.T) {
 	time.Sleep(500 * time.Millisecond)
 
 	//check file content
-	x, err := ioutil.ReadFile(expectedFile)
+	x, err := os.ReadFile(expectedFile)
 	if err == nil && len(x) > 0 {
 		t.Fatalf("file should be empty, got '%s'", x)
 	}
 
 	os.Remove("./crowdsec.log")
 	os.Remove(expectedFile)
-
 }
