@@ -4,24 +4,19 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"io/fs"
-	"math"
 	"os"
-	"os/exec"
-	"os/user"
 	"path/filepath"
-	"strconv"
 	"strings"
 	"sync"
-	"syscall"
 	"text/template"
 	"time"
 
-	"github.com/Masterminds/sprig"
+	"github.com/Masterminds/sprig/v3"
 	"github.com/crowdsecurity/crowdsec/pkg/csconfig"
 	"github.com/crowdsecurity/crowdsec/pkg/models"
 	"github.com/crowdsecurity/crowdsec/pkg/protobufs"
 	"github.com/crowdsecurity/crowdsec/pkg/types"
+	"github.com/google/uuid"
 	plugin "github.com/hashicorp/go-plugin"
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
@@ -29,7 +24,6 @@ import (
 	"gopkg.in/yaml.v2"
 )
 
-var testMode bool = false
 var pluginMutex sync.Mutex
 
 const (
@@ -37,6 +31,9 @@ const (
 	CrowdsecPluginKey     string = "CROWDSEC_PLUGIN_KEY"
 )
 
+// The broker is responsible for running the plugins and dispatching events
+// It receives all the events from the main process and stacks them up
+// It is as well notified by the watcher when it needs to deliver events to plugins (based on time or count threshold)
 type PluginBroker struct {
 	PluginChannel                   chan ProfileAlert
 	alertsByPluginName              map[string][]*models.Alert
@@ -55,12 +52,12 @@ type PluginBroker struct {
 type PluginConfig struct {
 	Type           string        `yaml:"type"`
 	Name           string        `yaml:"name"`
-	GroupWait      time.Duration `yaml:"group_wait"`
-	GroupThreshold int           `yaml:"group_threshold"`
-	MaxRetry       int           `yaml:"max_retry"`
-	TimeOut        time.Duration `yaml:"timeout"`
+	GroupWait      time.Duration `yaml:"group_wait,omitempty"`
+	GroupThreshold int           `yaml:"group_threshold,omitempty"`
+	MaxRetry       int           `yaml:"max_retry,omitempty"`
+	TimeOut        time.Duration `yaml:"timeout,omitempty"`
 
-	Format string `yaml:"format"` // specific to notification plugins
+	Format string `yaml:"format,omitempty"` // specific to notification plugins
 
 	Config map[string]interface{} `yaml:",inline"` //to keep the plugin-specific config
 
@@ -99,8 +96,10 @@ func (pb *PluginBroker) Kill() {
 	}
 }
 
-func (pb *PluginBroker) Run(tomb *tomb.Tomb) {
-	pb.watcher.Start(tomb)
+func (pb *PluginBroker) Run(pluginTomb *tomb.Tomb) {
+	//we get signaled via the channel when notifications need to be delivered to plugin (via the watcher)
+	pb.watcher.Start(&tomb.Tomb{})
+loop:
 	for {
 		select {
 		case profileAlert := <-pb.PluginChannel:
@@ -109,6 +108,7 @@ func (pb *PluginBroker) Run(tomb *tomb.Tomb) {
 		case pluginName := <-pb.watcher.PluginEvents:
 			// this can be ran in goroutine, but then locks will be needed
 			pluginMutex.Lock()
+			log.Tracef("going to deliver %d alerts to plugin %s", len(pb.alertsByPluginName[pluginName]), pluginName)
 			tmpAlerts := pb.alertsByPluginName[pluginName]
 			pb.alertsByPluginName[pluginName] = make([]*models.Alert, 0)
 			pluginMutex.Unlock()
@@ -118,13 +118,32 @@ func (pb *PluginBroker) Run(tomb *tomb.Tomb) {
 				}
 			}()
 
-		case <-tomb.Dying():
-			log.Info("killing all plugins")
-			pb.Kill()
-			return
+		case <-pluginTomb.Dying():
+			log.Infof("pluginTomb dying")
+			pb.watcher.tomb.Kill(errors.New("Terminating"))
+			for {
+				select {
+				case <-pb.watcher.tomb.Dead():
+					log.Info("killing all plugins")
+					pb.Kill()
+					break loop
+				case pluginName := <-pb.watcher.PluginEvents:
+					// this can be ran in goroutine, but then locks will be needed
+					pluginMutex.Lock()
+					log.Tracef("going to deliver %d alerts to plugin %s", len(pb.alertsByPluginName[pluginName]), pluginName)
+					tmpAlerts := pb.alertsByPluginName[pluginName]
+					pb.alertsByPluginName[pluginName] = make([]*models.Alert, 0)
+					pluginMutex.Unlock()
+
+					if err := pb.pushNotificationsToPlugin(pluginName, tmpAlerts); err != nil {
+						log.WithField("plugin:", pluginName).Error(err)
+					}
+				}
+			}
 		}
 	}
 }
+
 func (pb *PluginBroker) addProfileAlert(profileAlert ProfileAlert) {
 	for _, pluginName := range pb.profileConfigs[profileAlert.ProfileID].Notifications {
 		if _, ok := pb.pluginConfigByName[pluginName]; !ok {
@@ -157,7 +176,7 @@ func (pb *PluginBroker) loadConfig(path string) error {
 			continue
 		}
 
-		pluginConfigs, err := parsePluginConfigFile(configFilePath)
+		pluginConfigs, err := ParsePluginConfigFile(configFilePath)
 		if err != nil {
 			return err
 		}
@@ -176,6 +195,7 @@ func (pb *PluginBroker) loadConfig(path string) error {
 	return err
 }
 
+// checks whether every notification in profile has it's own config file
 func (pb *PluginBroker) verifyPluginConfigsWithProfile() error {
 	for _, profileCfg := range pb.profileConfigs {
 		for _, pluginName := range profileCfg.Notifications {
@@ -188,6 +208,18 @@ func (pb *PluginBroker) verifyPluginConfigsWithProfile() error {
 	return nil
 }
 
+// check whether each plugin in profile has it's own binary
+func (pb *PluginBroker) verifyPluginBinaryWithProfile() error {
+	for _, profileCfg := range pb.profileConfigs {
+		for _, pluginName := range profileCfg.Notifications {
+			if _, ok := pb.notificationPluginByName[pluginName]; !ok {
+				return fmt.Errorf("binary for plugin %s not found", pluginName)
+			}
+		}
+	}
+	return nil
+}
+
 func (pb *PluginBroker) loadPlugins(path string) error {
 	binaryPaths, err := listFilesAtPath(path)
 	if err != nil {
@@ -222,7 +254,7 @@ func (pb *PluginBroker) loadPlugins(path string) error {
 			if err != nil {
 				return err
 			}
-
+			data = []byte(os.ExpandEnv(string(data)))
 			_, err = pluginClient.Configure(context.Background(), &protobufs.Config{Config: data})
 			if err != nil {
 				return errors.Wrapf(err, "while configuring %s", pc.Name)
@@ -231,20 +263,20 @@ func (pb *PluginBroker) loadPlugins(path string) error {
 			pb.notificationPluginByName[pc.Name] = pluginClient
 		}
 	}
-	return err
+	return pb.verifyPluginBinaryWithProfile()
 }
 
 func (pb *PluginBroker) loadNotificationPlugin(name string, binaryPath string) (Notifier, error) {
+
 	handshake, err := getHandshake()
 	if err != nil {
 		return nil, err
 	}
-	cmd := exec.Command(binaryPath)
-	cmd.SysProcAttr, err = getProcessAtr(pb.pluginProcConfig.User, pb.pluginProcConfig.Group)
+	log.Debugf("Executing plugin %s", binaryPath)
+	cmd, err := pb.CreateCmd(binaryPath)
 	if err != nil {
-		return nil, errors.Wrap(err, "while getting process attributes")
+		return nil, err
 	}
-	cmd.SysProcAttr.Credential.NoSetGroups = true
 	pb.pluginMap[name] = &NotifierPlugin{}
 	l := log.New()
 	err = types.ConfigureLogger(l)
@@ -275,6 +307,7 @@ func (pb *PluginBroker) loadNotificationPlugin(name string, binaryPath string) (
 }
 
 func (pb *PluginBroker) pushNotificationsToPlugin(pluginName string, alerts []*models.Alert) error {
+	log.WithField("plugin", pluginName).Debugf("pushing %d alerts to plugin", len(alerts))
 	if len(alerts) == 0 {
 		return nil
 	}
@@ -296,9 +329,9 @@ func (pb *PluginBroker) pushNotificationsToPlugin(pluginName string, alerts []*m
 			},
 		)
 		if err == nil {
-			return err
+			return nil
 		}
-		log.WithField("plugin", pluginName).Errorf("%s error, retry num %d", err.Error(), i)
+		log.WithField("plugin", pluginName).Errorf("%s error, retry num %d", err, i)
 		time.Sleep(backoffDuration)
 		backoffDuration *= 2
 	}
@@ -306,7 +339,7 @@ func (pb *PluginBroker) pushNotificationsToPlugin(pluginName string, alerts []*m
 	return err
 }
 
-func parsePluginConfigFile(path string) ([]PluginConfig, error) {
+func ParsePluginConfigFile(path string) ([]PluginConfig, error) {
 	parsedConfigs := make([]PluginConfig, 0)
 	yamlFile, err := os.Open(path)
 	if err != nil {
@@ -321,7 +354,7 @@ func parsePluginConfigFile(path string) ([]PluginConfig, error) {
 			if err == io.EOF {
 				break
 			}
-			return []PluginConfig{}, fmt.Errorf("while decoding %s got error %s", path, err.Error())
+			return []PluginConfig{}, fmt.Errorf("while decoding %s got error %s", path, err)
 		}
 		parsedConfigs = append(parsedConfigs, pc)
 	}
@@ -337,41 +370,6 @@ func setRequiredFields(pluginCfg *PluginConfig) {
 		pluginCfg.TimeOut = time.Second * 5
 	}
 
-	if pluginCfg.GroupWait == time.Second*0 {
-		pluginCfg.GroupWait = time.Second * 1
-	}
-}
-
-func pluginIsValid(path string) error {
-	if testMode {
-		return nil
-	}
-	var details fs.FileInfo
-	var err error
-
-	// check if it exists
-	if details, err = os.Stat(path); err != nil {
-		return errors.Wrap(err, fmt.Sprintf("plugin at %s does not exist", path))
-	}
-
-	// check if it is owned by current user
-	currentUser, err := user.Current()
-	if err != nil {
-		return errors.Wrap(err, "while getting current user")
-	}
-	procAttr, err := getProcessAtr(currentUser.Username, currentUser.Username)
-	if err != nil {
-		return errors.Wrap(err, "while getting process attributes")
-	}
-	stat := details.Sys().(*syscall.Stat_t)
-	if stat.Uid != procAttr.Credential.Uid || stat.Gid != procAttr.Credential.Gid {
-		return fmt.Errorf("plugin at %s is not owned by %s user and group", path, currentUser.Username)
-	}
-
-	if (int(details.Mode()) & 2) != 0 {
-		return fmt.Errorf("plugin at %s is world writable, world writable plugins are invalid", path)
-	}
-	return nil
 }
 
 // helper which gives paths to all files in the given directory non-recursively
@@ -390,52 +388,12 @@ func listFilesAtPath(path string) ([]string, error) {
 	return filePaths, nil
 }
 
-func getPluginTypeAndSubtypeFromPath(path string) (string, string, error) {
-	pluginFileName := filepath.Base(path)
-	parts := strings.Split(pluginFileName, "-")
-	if len(parts) < 2 {
-		return "", "", fmt.Errorf("plugin name %s is invalid. Name should be like {type-name}", path)
-	}
-	return strings.Join(parts[:len(parts)-1], "-"), parts[len(parts)-1], nil
-}
-
-func getProcessAtr(username string, groupname string) (*syscall.SysProcAttr, error) {
-	u, err := user.Lookup(username)
-	if err != nil {
-		return nil, err
-	}
-	g, err := user.LookupGroup(groupname)
-	if err != nil {
-		return nil, err
-	}
-	uid, err := strconv.Atoi(u.Uid)
-	if err != nil {
-		return nil, err
-	}
-	if uid < 0 && uid > math.MaxInt32 {
-		return nil, fmt.Errorf("out of bound uid")
-	}
-	gid, err := strconv.Atoi(g.Gid)
-	if err != nil {
-		return nil, err
-	}
-	if gid < 0 && gid > math.MaxInt32 {
-		return nil, fmt.Errorf("out of bound gid")
-	}
-	return &syscall.SysProcAttr{
-		Credential: &syscall.Credential{
-			Uid: uint32(uid),
-			Gid: uint32(gid),
-		},
-	}, nil
-}
-
 func getUUID() (string, error) {
-	if d, err := os.ReadFile("/proc/sys/kernel/random/uuid"); err != nil {
+	uuidv4, err := uuid.NewRandom()
+	if err != nil {
 		return "", err
-	} else {
-		return string(d), nil
 	}
+	return uuidv4.String(), nil
 }
 
 func getHandshake() (plugin.HandshakeConfig, error) {
